@@ -9,6 +9,7 @@ use Regexp::Assemble::Compressed;
 use HTML::TableParser::Grid;
 use Spreadsheet::ParseExcel;
 use File::Basename;
+use File::Path;
 use Getopt::Long;
 
 our $DEBUG;
@@ -34,44 +35,57 @@ main();
 
 sub main {
     my %task_map = (
-        Class1  => +{ function => 'class' },
-        Class2  => +{ skip => 1 },
-        Phs     => +{
+        Class1   => +{ function => 'class' },
+        Class2   => +{ skip => 1 },
+        Phs      => +{
             function    => 'fixed_pref',
             prefix      => '070',
             test_suffix => '12345',
         },
-        Pager   => +{
+        Pager    => +{
             function    => 'fixed_pref',
             prefix      => '020',
             test_suffix => '12345',
         },
-        Q2      => +{
+        Q2       => +{
             function    => 'fixed_pref',
-            filename    => 'jyoho_number.htm',
             prefix      => '0990',
-            test_suffix      => '123',
+            test_suffix => '123',
+            filename    => 'jyoho_number.xls',
         },
-        Fmc     => +{
+        Fmc      => +{
             function    => 'fixed_pref',
             prefix      => '060',
-            test_suffix => '1234',
         },
-        Upt     => +{ function    => 'upt' },
-        United  => +{
+        Upt      => +{ function    => 'upt' },
+        United   => +{
             function    => 'fixed_pref',
-            filename    => 'toitu_number.htm',
             prefix      => '0570',
             test_suffix => '123',
+            filename    => 'toitu_number.xls',
         },
-        Ipphone => +{
+        Ipphone  => +{
             function    => 'fixed_pref',
-            filename    => 'ip_number.htm',
             prefix      => '050',
-            test_suffix => '1234',
+            filename    => 'ip_number.xls',
+        },
+        Mobile   => +{
+            function    => 'fixed_pref',
+            prefix      => [qw(080 090)],
+            test_suffix => '12345',
+            filename    => [qw(mobile80.xls mobile90.xls)],
+        },
+        Freedial => +{
+            function    => 'fixed_pref',
+            prefix      => [qw(0120 0800)],
+            test_suffix => +{ '0120' => '123', '0800' => '1234' },
+            filename    => [qw(mobile0120.xls mobile0800.xls)],
         },
     );
 
+    unless (-d $STOREDIR) {
+        mkpath($STOREDIR) or die $!;
+    }
     opendir my $dh, $TABLEDIR or die "$TABLEDIR: $!";
     for my $file (readdir $dh) {
         next unless $file =~ /^(\w+)\.pm$/;
@@ -95,46 +109,83 @@ sub fixed_pref {
     my($class, $param) = @_;
     my $lc_class = lc($class);
     my $filename = "$TABLEDIR/$class.pm";
-    my $file = $param->{filename} || "$lc_class.htm";
-    _warn($file);
-    $file = "$STOREDIR/$file";
-    my $res = http_get_file($file);
-    unless ($res == 200) {
-        _warn("fail to get new file: $file ($res)");
-        return;
+    my @files = ();
+    if ($param->{filename}) {
+        @files = ref($param->{filename}) eq 'ARRAY' ? 
+            @{$param->{filename}} : ($param->{filename});
+    }
+    else {
+        @files = ("$lc_class.xls");
+    }
+    my @prefixes = ref($param->{prefix}) eq 'ARRAY' ?
+        @{$param->{prefix}} : ($param->{prefix});
+    my %regexp_table =
+        map { $_ => Regexp::Assemble::Compressed->new } @prefixes;
+    my $prefix_re;
+    if (@prefixes == 1) {
+        $prefix_re = $prefixes[0];
+    }
+    else {
+        my $re = Regexp::Assemble::Compressed->new;
+        for my $prefix (@prefixes) {
+            $re->add($prefix);
+        }
+        ($prefix_re = $re->re) =~ s/^\(\?-xism:/(?:/;
+    }
+    my @rows_list = ();
+    my @cols_list = ();
+    my @column_values_list = ();
+    for my $file (@files) {
+        _warn($file);
+        $file = "$STOREDIR/$file";
+        http_get_file($file) or die "HTTP failed";
+        my($rows, $cols, $column_values) = parse_excel($file);
+        push @rows_list, $rows;
+        push @cols_list, $cols;
+        push @column_values_list, $column_values;
     }
-    my $prefix = $param->{prefix};
-    my($rows, $cols, $column_values) = table_parse($file, $prefix);
-
     open my $fh, '>', $filename or die "$filename: $!";
     print $fh table_class_header($class);
 
     my @ok = ();
     my @ng = ();
-    my $test_suffix   = $param->{test_suffix};
-    my $regexp_suffix = '\d{' . length($test_suffix) . '}';
 
     my $re = Regexp::Assemble::Compressed->new;
-    for my $row (sort { $a <=> $b } keys %$rows) {
-        for my $col (sort { $a <=> $b } keys %$cols) {
-            my $number = sprintf '%s%s', $rows->{$row}, $cols->{$col};
-            _warn($number . ("x" x (length $test_suffix)));
-            my $value = $column_values->{$row}{$col};
-            $number =~ s/^$prefix//;
-            if (!defined $value || $value =~ /^(\s|-)*$/) {
-                push @ng, "$prefix ${number}${test_suffix}";
-                next;
+    for my $i (0 .. $#rows_list) {
+        my $rows = $rows_list[$i];
+        my $cols = $cols_list[$i];
+        my $column_values = $column_values_list[$i];
+        for my $row (sort { $a <=> $b } keys %$rows) {
+            for my $col (sort { $a <=> $b } keys %$cols) {
+                my $number = sprintf '%s%s', $rows->{$row}, $cols->{$col};
+                my $value = $column_values->{$row}{$col};
+                my $orig_number = $number;
+                $number =~ s/^($prefix_re)//;
+                my $prefix = $1;
+                my $test_suffix =
+                    ref($param->{test_suffix}) eq 'HASH' ?
+                        $param->{test_suffix}{$prefix} :
+                            ($param->{test_suffix} || '1234');
+                my $regexp_suffix = '\d{' . length($test_suffix) . '}';
+                _warn($orig_number . ("x" x (length $test_suffix)));
+                if (!defined $value || $value =~ /^(\s|-)*$/) {
+                    push @ng, "$prefix ${number}${test_suffix}";
+                    next;
+                }
+                else {
+                    push @ok, "$prefix ${number}${test_suffix}";
+                }
+                my $re = $regexp_table{$prefix};
+                $re->add($number . $regexp_suffix);
             }
-            else {
-                push @ok, "$prefix ${number}${test_suffix}";
-            }
-            $re->add($number . $regexp_suffix);
         }
     }
-
-    (my $table_prefix = $param->{prefix}) =~ s/^0//;
-    (my $regexp = $re->re) =~ s/^\(\?-xism:/(?:/;
-    printf $fh "    $table_prefix => '%s',\n", compress($regexp);
+    for my $prefix (@prefixes) {
+        my $re = $regexp_table{$prefix};
+        (my $regexp = $re->re) =~ s/^\(\?-xism:/(?:/;
+        (my $table_prefix = $prefix) =~ s/^0//;
+        printf $fh "    $table_prefix => '%s',\n", compress($regexp);
+    }
     printf $fh table_class_footer();
     close $fh;
 
@@ -150,24 +201,24 @@ sub home {
     my @ng = ();
     my $modified;
 
+    no warnings 'uninitialized';
     for my $num (1 .. 9) {
         my $file = sprintf 'fixed_%d.xls', $num;
         _warn($file);
         $file = "$STOREDIR/$file";
-        my $res = http_get_file($file);
-        unless ($res == 200) {
-            _warn("fail to get new file: $file ($res)");
-            next;
-        }
+        http_get_file($file) or die "HTTP failed";
         $modified = 1;
-        my $excel = Spreadsheet::ParseExcel::Workbook->Parse($file);
-        my $sheet = shift @{$excel->{Worksheet}};
-        $sheet->{MaxRow} ||= $sheet->{MinRow};
-        for my $row ($sheet->{MinRow} .. $sheet->{MaxRow}) {
-            my $pref = $sheet->{Cells}[$row][3]->{Val};
+        my $parser = Spreadsheet::ParseExcel->new;
+        my $workbook = $parser->parse($file);
+        my $sheet = ($workbook->worksheets)[0];
+        my @row_range = $sheet->row_range;
+        for my $row ($row_range[0] .. $row_range[1]) {
+            my $cell = $sheet->get_cell($row, 3);
+            next unless defined $cell;
+            my $pref = $cell->value;
             next unless defined $pref && $pref =~ s/^0//;
-            my $local_pref = $sheet->{Cells}[$row][4]->{Val};
-            my $status = encode('utf-8', $sheet->{Cells}[$row][6]->Value);
+            my $local_pref = $sheet->get_cell($row, 4)->value;
+            my $status = encode('utf-8', $sheet->get_cell($row, 6)->value);
             unless ($status =~ /(?:使用中|使用予定)/) {
                 push @ng, sprintf '0%s %s1234', $pref, $local_pref;
                 next;
@@ -195,40 +246,37 @@ sub home {
 }
 
 sub class {
-    my $file = 'company1.htm';
+    my $file = 'company1.xls';
     _warn($file);
-    $file = "$STOREDIR/$file";
-    my $res = http_get_file($file);
-    unless ($res == 200) {
-        _warn("fail to get new file: $file ($res)");
-        return;
-    }
-    my $prefix = qr<00(?:[3-8]|2\d|91\d)?$>;
-    my($rows, $cols, $column_values) = table_parse($file, $prefix);
+    $file = "$STOREDIR/company1.xls";
+    http_get_file($file) or die "HTTP failed";
 
-    # formerly, the numbers that's prefixed by "009" are called class2.
-    # it's left for backward compatibility.
-    my @class1_rows = ();
-    my @class2_rows = ();
-    no warnings 'uninitialized';
-    for my $row (sort { $a <=> $b } %$rows) {
-        if ($rows->{$row} =~ /^009/) {
-            push @class2_rows, $row;
-        }
-        else {
-            push @class1_rows, $row;
-        }
-    }
+    my($rows, $cols, $column_values) = parse_excel($file);
 
     my @ok = ();
     my @ng = ();
-    my $class1_file = "$TABLEDIR/Class1.pm";
-    my $class1_test = "$TESTDIR/class1.t";
 
-    no warnings 'uninitialized';
-    open my $fh, '>', $class1_file or die "$class1_file: $!";
+    my @rows1 = ();
+    my @rows2 = ();
+    my $start_class2 = 0;
+    for my $row (sort { $a <=> $b } keys %$rows) {
+        if ($start_class2) {
+            push @rows2, $row;
+        }
+        else {
+            if ($rows->{$row} !~ /^\d+$/) {
+                $start_class2 = 1;
+                next;
+            }
+            push @rows1, $row;
+        }
+    }
+
+    my $filename = "$TABLEDIR/Class1.pm";
+
+    open my $fh, '>', $filename or die "$filename: $!";
     print $fh table_class_header('Class1');
-    for my $row (@class1_rows) {
+    for my $row (@rows1) {
         for my $col (sort { $a <=> $b } keys %$cols) {
             next unless $rows->{$row};
             my $prefix = sprintf '%s%s', $rows->{$row}, $cols->{$col};
@@ -253,13 +301,14 @@ sub class {
 
     @ok = ();
     @ng = ();
-    my $class2_file = "$TABLEDIR/Class2.pm";
-    my $class2_test = "$TESTDIR/class2.t";
 
-    open $fh, '>', $class2_file or die "$class2_file: $!";
+    $filename = "$TABLEDIR/Class2.pm";
+
+    open $fh, '>', $filename or die "$filename: $!";
     print $fh table_class_header('Class2');
-    for my $row (@class2_rows) {
+    for my $row (@rows2) {
         for my $col (sort { $a <=> $b } keys %$cols) {
+            next unless $rows->{$row};
             my $prefix = sprintf '%s%s', $rows->{$row}, $cols->{$col};
             _warn("${prefix}xxxxxxxx");
             my $value = $column_values->{$row}{$col};
@@ -281,165 +330,6 @@ sub class {
     make_test('class2', \@ok, \@ng);
 }
 
-sub mobile {
-    my $filename = "$TABLEDIR/Mobile.pm";
-    my $file = 'mobile80.htm';
-    _warn($file);
-    $file = "$STOREDIR/$file";
-    my $res = http_get_file($file);
-    unless ($res == 200) {
-        _warn("fail to get new file: $file ($res)");
-        return;
-    }
-
-    my $rows = {};
-    my $cols = {};
-    my $column_values = {};
-
-    ($rows, $cols, $column_values) = table_parse($file, '080');
-
-    open my $fh, '>', $filename or die "$filename: $!";
-    print $fh table_class_header('Mobile');
-
-    my @ok = ();
-    my @ng = ();
-    my $re = Regexp::Assemble::Compressed->new;
-    for my $row (sort { $a <=> $b } keys %$rows) {
-        for my $col (sort { $a <=> $b } keys %$cols) {
-            my $number = sprintf '%s%s', $rows->{$row}, $cols->{$col};
-            _warn("${number}xxxxx");
-            my $value = $column_values->{$row}{$col};
-            $number =~ s/^080//;
-            if (!defined $value || $value =~ /^(\s|-)*$/) {
-                push @ng, "080 ${number}12345";
-                next;
-            }
-            else {
-                push @ok, "080 ${number}12345";
-            }
-            $re->add($number . '\d{5}');
-        }
-    }
-    (my $regexp = $re->re) =~ s/^\(\?-xism:/(?:/;
-    printf $fh "    80 => '%s',\n", compress($regexp);
-
-    $file = 'mobile90.htm';
-    _warn($file);
-    $file = "$STOREDIR/$file";
-    $res = http_get_file($file);
-    unless ($res == 200) {
-        _warn("fail to get new file: $file ($res)");
-        return;
-    }
-
-    ($rows, $cols, $column_values) = table_parse($file, '090');
-
-    $re = Regexp::Assemble::Compressed->new;
-    for my $row (sort { $a <=> $b } keys %$rows) {
-        for my $col (sort { $a <=> $b } keys %$cols) {
-            my $number = sprintf '%s%s', $rows->{$row}, $cols->{$col};
-            _warn("${number}xxxxx");
-            my $value = $column_values->{$row}{$col};
-            $number =~ s/^090//;
-            if ($value =~ /^(\s|-)*$/) {
-                push @ng, "090 ${number}12345";
-                next;
-            }
-            else {
-                push @ok, "090 ${number}12345";
-            }
-            $re->add($number . '\d{5}');
-        }
-    }
-    ($regexp = $re->re) =~ s/^\(\?-xism:/(?:/;
-    printf $fh "    90 => '%s',\n", compress($regexp);
-    printf $fh table_class_footer();
-    close $fh;
-
-    make_test('mobile', \@ok, \@ng);
-}
-
-sub freedial {
-    my $filename = "$TABLEDIR/Freedial.pm";
-    my $file = 'mobile0120.htm';
-    _warn($file);
-    $file = "$STOREDIR/$file";
-    my $res = http_get_file($file);
-    unless ($res == 200) {
-        _warn("fail to get new file: $file ($res)");
-        return;
-    }
-
-    my $rows = {};
-    my $cols = {};
-    my $column_values = {};
-
-    open my $in, $file or die "$file: $!";
-    my $html = do { local $/; <$in> };
-    close $in;
-
-    ($rows, $cols, $column_values) = table_parse($file, '0120');
-
-    open my $fh, '>', $filename or die "$filename: $!";
-    print $fh table_class_header('Freedial');
-
-    my @ok = ();
-    my @ng = ();
-    my $re = Regexp::Assemble::Compressed->new;
-    for my $row (sort { $a <=> $b } keys %$rows) {
-        for my $col (sort { $a <=> $b } keys %$cols) {
-            my $number = sprintf '%s%s', $rows->{$row}, $cols->{$col};
-            _warn("${number}xxx");
-            my $value = $column_values->{$row}{$col};
-            $number =~ s/^0120//;
-            if ($value =~ /^(\s|-)*$/) {
-                push @ng, "0120 ${number}123";
-                next;
-            }
-            else {
-                push @ok, "0120 ${number}123";
-            }
-            $re->add($number . '\d{3}');
-        }
-    }
-    (my $regexp = $re->re) =~ s/^\(\?-xism:/(?:/;
-    printf $fh "    120 => '%s',\n", compress($regexp);
-
-    $file = 'mobile0800.htm';
-    _warn($file);
-    $file = "$STOREDIR/$file";
-    $res = http_get_file($file);
-    unless ($res == 200) {
-        _warn("fail to get new file: $file ($res)");
-        return;
-    }
-    ($rows, $cols, $column_values) = table_parse($file, '0800');
-
-    $re = Regexp::Assemble::Compressed->new;
-    for my $row (sort { $a <=> $b } keys %$rows) {
-        for my $col (sort { $a <=> $b } keys %$cols) {
-            my $number = sprintf '%s%s', $rows->{$row}, $cols->{$col};
-            _warn("${number}xxxx");
-            my $value = $column_values->{$row}{$col};
-            $number =~ s/^0800//;
-            if (!defined $value || $value =~ /^(\s|-)*$/) {
-                push @ng, "0800 ${number}1234";
-                next;
-            }
-            else {
-                push @ok, "0800 ${number}1234";
-            }
-            $re->add($number . '\d{4}');
-        }
-    }
-    ($regexp = $re->re) =~ s/^\(\?-xism:/(?:/;
-    printf $fh "    800 => '%s',\n", compress($regexp);
-    printf $fh table_class_footer();
-    close $fh;
-
-    make_test('freedial', \@ok, \@ng);
-}
-
 sub upt {
     my $filename = "$TABLEDIR/Upt.pm";
     open my $fh, '>', $filename or die "$filename: $!";
@@ -462,61 +352,64 @@ END
     make_test('upt', [], []);
 }
 
-sub table_parse {
-    my($file, $prefix) = @_;
-    open my $in, $file or die "$file: $!";
-    my $html = do { local $/; <$in> };
-    close $in;
-    # HTML::TableParser::Grid does not support parsing <th> tag.
-    $html =~ s{(</?)th(>)}{$1td$2}gi;
-
-    my $parser = HTML::TableParser::Grid->new($html, 0);
+sub parse_excel {
+    my $file = shift;
+    my $parser = Spreadsheet::ParseExcel->new;
+    my $workbook = $parser->parse($file);
+    my $sheet = ($workbook->worksheets)[0];
+    my($row_from, $row_to) = $sheet->row_range;
+    my($col_from, $col_to) = $sheet->col_range;
     my %rows = ();
     my %cols = ();
     my %column_values = ();
-
-    no warnings 'uninitialized';
-    for my $row (0 .. $parser->num_rows) {
+    my $start_reading = 0;
+    for my $row ($row_from .. $row_to) {
         my $read_header = 0;
-        for my $col (0 .. $parser->num_columns) {
-            my $value = convert_value($parser->cell($row, $col));
-            $column_values{$row}{$col} = $value;
+        for my $col ($col_from .. $col_to) {
             if ($col == 0) {
-                if ($value eq '番号') {
-                    # probably, they're wrote on the table header
-                    $read_header = 1;
+                my $cell = $sheet->get_cell($row, $col);
+                my $value = $cell ? convert_value($cell->value) : '';
+                if ($start_reading) {
+                    next unless length $value;
+                    $rows{$row} = $value =~ /^0/ ? $value : '0' . $value;
                 }
-                elsif ($value =~ /^$prefix/) {
-                    $rows{$row} = $value;
+                else {
+                    if ($value eq '番号') {
+                        $read_header   = 1;
+                        $start_reading = 1;
+                        next;
+                    }
                 }
+                next;
             }
-            elsif ($read_header) {
+            last unless $start_reading;
+            my $cell = $sheet->get_cell($row, $col);
+            my $value = $cell ? convert_value($cell->value) : '';
+            $column_values{$row}{$col} = $value;
+            if ($read_header) {
                 if ($value =~ /^\d$/) {
                     $cols{$col} = $value;
                 }
+                next;
             }
         }
     }
-
-    return(\%rows, \%cols, \%column_values);
+    return (\%rows, \%cols, \%column_values);
 }
 
 sub http_get_file {
     my $file = shift;
     my $uri = basename($file);
+    my($ext) = $uri =~ /\.([^\.]+)$/;
     my $url =
-        sprintf 'http://www.soumu.go.jp/johotsusintokei/tel_number/%s', $uri;
-    return LWP::Simple::mirror($url, $file);
-}
-
-sub convert_value {
-    my $value = shift;
-    no warnings 'uninitialized';
-    return unless length $value;
-    $value = decode('shift_jis', $value);
-    use utf8;
-    $value =~ tr/０-９Ａ-Ｚａ-ｚ　！”＃＄％＆’（）＊＋，−．／：；＜＝＞？＠［¥］＾＿‘｛｜｝〜/0-9A-Za-z !"#$%&'()*+,-.\/:;<=>?@[\\]^_`{|}~/;
-    return encode('utf-8', $value);
+        sprintf
+            'http://www.soumu.go.jp/main_sosiki/joho_tsusin/top/tel_number/%s/%s',
+                $ext, $uri;
+    _warn($url);
+    my $res = LWP::Simple::mirror($url, $file);
+    return 1 if $res == 200 || $res == 304;
+    _warn("fail to get new file: $file ($res)");
+    return;
 }
 
 sub compress { # makes regexp more compressed
@@ -537,6 +430,13 @@ sub compress { # makes regexp more compressed
     return $regexp;
 }
 
+sub convert_value {
+    my $cell_value = shift;
+    $cell_value =~ tr/\x{3000}\x{FF01}-\x{FF5E}/\x20\x21-\x7E/;
+    $cell_value =~ tr/\x{201D}\x{2019}\x{FFE5}\x{2018}\x{301C}/"'\\`~/;
+    $cell_value =~ tr/\x{2010}-\x{2015}\x{2212}/\-\-\-\-\-\-\-/;
+    return encode('utf-8', $cell_value);
+}
 
 sub table_class_header {
     my $name = shift;
